﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

// check whether a given int is a prime number
let isPrime x =
    let mutable result = true 
    for div in 2..(x/2) do
        if x % div = 0 then
            result <- false
    result
    
// a function that counts all prime numbers up to a given max
let countPrimes max = 
    let mutable result = 0
    for v in 1..max do
        if isPrime v then
            result <- result + 1
    result
printfn "%d" (countPrimes 20)

// Given the following list, outputs this:
//
// Willy drives a Mercedes
// Harry drives a Vauxhall
// Anna doesn't have a car
//
let drivers = 
    [
        ("Willy", Some("Mercedes"));
        ("Harry", Some("Vauxhall"));
        ("Anna", None)
    ]

let outputDrivers (l: (string * string option) list) = 
    for record in l do
        let name, carOption = record
        if carOption.IsSome then
            printfn "%s drives a %s" name carOption.Value
        else
            printfn "%s doesn't have a car" name
    
let outputDriversWithMatch l = 
    for (name, carOption) in l do
        match name, carOption with
        | _, Some(car) -> printfn "%s drives a %s" name car
        | _, None -> printfn "%s doesnt' have a car" name
        
outputDrivers drivers
outputDriversWithMatch drivers

// Given this list of customers and their sales volume
let customers = 
    [
        ("Billy Bott", 3531.45m);
        ("Jimmy Jones", 7400m);
        ("Sylvia Simmons", 1219.99m)
    ]
    
// a function to return a list of all the sales volumes

let customerSalesVolumes customers = 
    let mutable result = []
    for customer in customers do
        let name, salesVolume = customer
        result <- salesVolume :: result
    result

// a function to calculate the total sales volume from
// the list of customers
let calcTotalSalesVolume customers =
    let salesVolumes = customerSalesVolumes customers
    let mutable result = 0m
    for sv in salesVolumes do
        result <- result + sv
    result
   
printfn "Total sales volume is %M" (calcTotalSalesVolume customers)

// Nicer solutions to these problems would use 
// standard higher order functions. 

let customerSalesVolumesFunctional customers = 
    List.map (fun (name, sv) -> sv) customers

let calcTotalSalesVolumeFunctional customers = 
    List.fold (fun r (name, sv) -> r + sv) 0m customers
    
printfn "Total sales volume is %M" (calcTotalSalesVolumeFunctional customers)
