// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

open System
open System.Collections
open System.Collections.Generic

open System.Linq
open System.Linq.Expressions

open Microsoft.FSharp.Linq
open Microsoft.FSharp.Linq.Query
open Microsoft.FSharp.Linq.QuotationEvaluation

let outputList list = 
    for x in list do printfn "%A " x
    printf "\n"
    
type Customer = 
    { Name: string;
      Age: int;
      City: string;
      SalesVolume: decimal }
      
let customers = 
    [ { Name = "Bobby"; Age = 32; City = "Bristol"; SalesVolume = 2075.87M };
      { Name = "Harry"; Age = 37; City = "Birmingham"; SalesVolume = 122.54M };
      { Name = "Karen"; Age = 45; City = "London"; SalesVolume = 324.78M };
      { Name = "Lisa"; Age = 19; City = "Manchester"; SalesVolume = 837.00M };
      { Name = "Karl"; Age = 28; City = "London"; SalesVolume = 16.88M };
      { Name = "Susan"; Age = 25; City = "Dover"; SalesVolume = 97.13M } ]

// Calling a standard LINQ to Objects function
let where (f: 'a -> bool) s = Enumerable.Where(s, f)

let customersOver30 = 
    customers 
    |> where (fun c -> c.Age > 30)

outputList customersOver30

// Converting quotations into expressions
let quot = <@ new Func<_,_>(fun x -> x + 3) @>

let exp = quot.ToLinqExpression() :?> Expression<Func<int, int>>

ExpressionDumper.Output exp

printfn "%d" (exp.Compile().Invoke(20))

// Btw - this does *not* work because the quotation 
// evaluation mechanism "wraps" the Expression in an 
// additional MethodCallExpression
let quot2 = <@ fun x -> x + 3 @>
let exp2 = quot2.ToLinqExpression()
ExpressionDumper.Output exp2


// Querying through IQueryable
let customersQueryable = Queryable.AsQueryable<Customer>(customers |> List.to_seq)

let allCustomers = query <@ seq { for c in customersQueryable do
                                  yield (c.Name, c.Age) }  @>
outputList allCustomers

let someCustomers = query <@ seq { for c in customersQueryable do
                                    if c.Age > 30 then
                                        yield c } @>
outputList someCustomers

Console.ReadLine() |> ignore