﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

open System
open System.IO

// creating a file with some text in it 
using (File.CreateText("file.txt"))
    (fun file -> 
        file.WriteLine("// filterList uses a recursive approach to iterate over ")
        file.WriteLine("// a list, applies the given function to each element and ")
        file.WriteLine("// constructs a result list that contains only those elements")
        file.WriteLine("// of the original list for which the function returned true.")
        file.WriteLine("let rec filterList f (l: 'a list) = ")
        file.WriteLine("    match l with")
        file.WriteLine("    | [] -> []")
        file.WriteLine("    | hd :: tl -> ")
        file.WriteLine("        (if f hd then [hd] else []) @ (filterList f tl)")
    )

// a function "readFile" that reads the file line by line
// and returns a sequence of the line values
let readFile path = 
    seq {
        use file = File.OpenText(path)
        while not file.EndOfStream do
            yield file.ReadLine()
    }
    
// a function "filter" that takes a filter function
// and a sequence as its parameters. Then it iterates over the 
// source sequence and executes the given function for each element.
// All elements for which the function returns true become
// part of a result sequence.
let filter f s = 
    seq {
        for e in s do
            if f e then
                yield e
    }

// Using the functions readFile and filter to implement
// a filter for the file content. 
// Example: showing all the comment lines, i.e. those starting with //
let commentLines = 
    "file.txt" 
    |> readFile
    |> filter (fun l -> l.StartsWith("//"))

for l in commentLines do
    Console.WriteLine l
