﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

open Helpers

// This references the sequence data source defined in the
// helper library
let data = Data.countryInfoList

// calculate the total population over all countries
let totalPopulation = 
    data
    |> Seq.map (fun c -> c.Population)
    |> Seq.fold (fun (r: uint64) x -> r + uint64(x)) 0UL

printfn "Total population: %d" totalPopulation

// calculate the average population over all countries.
let averagePopulation = 
    data
    |> Seq.map (fun c -> c.Population)
    |> Seq.fold (fun (count, (popAccum: uint64)) pop -> 
                    (count + 1, popAccum + uint64(pop))) (0, 0UL)
    |> (fun (count, population) -> population / uint64(count))

printfn "Average population: %d" averagePopulation

// of course there are also the Seq.average and Seq.average_by functions
// these don't work with int though
let averagePopulationBoring = 
    data
    |> Seq.averageBy (fun c -> float(c.Population))

printfn "Average population: %f" averagePopulationBoring

// output a list of countries that is grouped and 
// sorted by millions of population, showing also the average 
// area per group

let showMillionsGroups() = 
    let groups = 
        data 
        |> Seq.groupBy (fun c -> c.Population/1000000)
        |> Seq.sortBy (fun g -> fst g)
    for g in groups do
        let averageArea = Seq.averageBy (fun (c:Data.CountryInfo) -> float(c.AreaKM2)) (snd g)
        printfn "Group %d millions, average Area: %f" (fst g) averageArea
        for ci in (snd g) do
            printfn "    Country: %s, Area: %d, Population: %d" ci.Name ci.AreaKM2 ci.Population
            
showMillionsGroups()
            