﻿#light

open System

// The following function "doSomeStuff" uses a 
// helper function "cleanup" to create an object that 
// can be used with the "use" keyword, passing in a function 
// that can perform general cleanup work. Through the mechanism
// of "use", the cleanup function is automatically called when
// "c" goes out of scope. 

let cleanup cu = { new IDisposable with member x.Dispose() = cu() }

let doSomeStuff() = 
    use c = cleanup (fun _ -> printfn "Doing cleanup")
    printfn "Doing stuff in the function"

doSomeStuff()

// A C# class hierarchy in F#:
//
//  public abstract class Fish {
//    public Fish(string name) {
//      this.name = name;
//    }
//    private string name;
//    public string Name {
//      get { return name; }
//    }
//    public abstract void Swim( );
//  }
//  public interface ISmell {
//    void Smell( );
//  }
//  public class Tuna : Fish {
//    public Tuna() : base("Tuna") { }
//    public override void Swim( ) {
//      // Tuna's specific swimming method
//    }
//  }
//  public abstract class Shark : Fish, ISmell {
//    public Shark() : base("Shark") { }
//    public abstract void Smell( );
//  }

[<AbstractClass>]
type Fish(name:string) =
    member x.Name = name
    abstract Swim: unit -> unit
    
type ISmell =
    abstract Smell: unit -> unit

type Tuna() = 
    inherit Fish("Tuna")
    default x.Swim() = printfn "Tuna swimming"
    
[<AbstractClass>]
type Shark() = 
    inherit Fish("Shark")
    abstract Smell: unit -> unit
    interface ISmell with
        member x.Smell() = x.Smell()
