﻿#light

open System
open System.Collections.Generic

// The following C# classes in F#:
//
//  public class Product {
//    private string name;
//    public string Name {
//      get { return name; }
//      set { name = value; }
//    }
//
//    private int stockLevel;
//    public int StockLevel {
//      get { return stockLevel; }
//      set { stockLevel = value; }
//    }
//  }
//  public class OrderLine {
//    private Product product;
//    public Product Product {
//      get { return product; }
//      set { product = value; }
//    }
//    private int count;
//    public int Count {
//      get { return count; }
//      set { count = value; }
//    }
//  }
//  public class Order {
//    private DateTime date = DateTime.Now;
//    public DateTime Date {
//      get { return date; }
//      set { date = value; }
//    }
//    private List<OrderLine> orderLines;
//    public List<OrderLine> OrderLines {
//      get { return orderLines; }
//      set { orderLines = value; }
//    }
//  }
//
// Plus some initialization code that creates the same
// objects from the classes as this C# code:
// 
//  var rubberChicken = new Product { Name = "Rubber Chicken", StockLevel = 50 };
//  var pulley = new Product { Name = "Pulley", StockLevel = 30 };
//  var order = new Order {
//    Date = DateTime.Now,
//    OrderLines = new List<OrderLine> {
//      new OrderLine { Product = rubberChicken, Count = 11 },
//      new OrderLine { Product = pulley, Count=11}
//    }
//  };

type Product() = 
    let name = ref ""
    let stockLevel = ref 0
    member x.Name with get() = !name and set(v) = name := v
    member x.StockLevel with get() = !stockLevel and set(v) = stockLevel := v

type OrderLine() = 
    let product = ref (Unchecked.defaultof<Product>)
    let count = ref 0
    member x.Product with get() = !product and set(v) = product := v
    member x.Count with get() = !count and set(v) = count := v

type OrderLineWithOption() = 
    let product: Product option ref = ref None
    let count = ref 0
    member x.Product with get() = !product and set(v) = product := v
    member x.Count with get() = !count and set(v) = count := v

type Order() = 
    let date = ref DateTime.Now
    let orderLines = ref (new List<OrderLine>())
    member x.Date with get() = !date and set(v) = date := v
    member x.OrderLines with get() = !orderLines and set(v) = orderLines := v

type OrderWithOption() = 
    let date = ref DateTime.Now
    let orderLines = ref (new List<OrderLineWithOption>())
    member x.Date with get() = !date and set(v) = date := v
    member x.OrderLines with get() = !orderLines and set(v) = orderLines := v

let rubberChicken = Product(Name = "Rubber Chicken", StockLevel = 50)
let pulley = Product(Name = "Pulley", StockLevel = 30)
let order = 
    Order(Date = DateTime.Now,
        OrderLines = new List<OrderLine>(
            seq {
                yield OrderLine(Product = rubberChicken, Count = 11)
                yield OrderLine(Product = pulley, Count = 11)
            }
        ))

let orderWithOption = 
    OrderWithOption(Date = DateTime.Now,
        OrderLines = new List<OrderLineWithOption>(
            seq {
                yield OrderLineWithOption(Product = Some(rubberChicken), Count = 11)
                yield OrderLineWithOption(Product = Some(pulley), Count = 11)
            }
        ))
